@@ -1,23 +1,24 @@
 import ctypes
 import json
 import jwt
-from typing import Dict, Callable, Any
-from django.http import JsonResponse, HttpRequest, HttpResponse
+import os
+import orjson
+from django.http import JsonResponse
 from django.utils.module_loading import import_string
 from django.conf import settings
 from . import ffi
 
 
 class Query:
-    instance: int
-    handle: int
-    query: bytes
-
-    def __init__(self, instance: int, query: str):
+    def __init__(self, instance, query):
+        self.handle = 0
         self.instance = instance
-        self.query = str.encode(query)
 
-    def process_request(self, auth: str) -> (Dict, Dict):
+        self.query = b''
+        if query:
+            self.query = str.encode(query)
+
+    def process_request(self, token):
         resp_input = ctypes.create_string_buffer(self.query)
 
         output_ptr = ctypes.c_char_p()
@@ -26,16 +27,18 @@ class Query:
         status_ptr = ctypes.c_char_p()
         status_len = ctypes.c_int()
 
-        auth = ctypes.create_string_buffer(b'{"jwt":"%s"}' % str.encode(auth))
+        auth = b''
+        if token:
+            auth = b'{"jwt":"%s"}' % str.encode(token)
 
         self.handle = ffi.process_request(self.instance,
-                                          auth, len(auth),
-                                          resp_input, len(resp_input),
+                                          ctypes.create_string_buffer(auth), len(auth),
+                                          resp_input, len(self.query),
                                           ctypes.byref(output_ptr), ctypes.byref(output_len),
                                           ctypes.byref(status_ptr), ctypes.byref(status_len))
 
-        output_dict: Dict = {}
-        status_dict: Dict = {}
+        output_dict = {}
+        status_dict = {}
 
         if output_len.value:
             output_dict = json.loads(output_ptr.value[:output_len.value].decode("utf-8"))
@@ -48,7 +51,7 @@ class Query:
 
         return output_dict, status_dict
 
-    def process_response(self, resp_body: bytes) -> Dict | None:
+    def process_response(self, resp_body):
         if self.handle == 0:
             return None
 
@@ -62,7 +65,7 @@ class Query:
             ctypes.byref(output_ptr), ctypes.byref(output_len)
         )
 
-        output_dict: Dict[str:Any] = {}
+        output_dict = {}
 
         if output_len.value:
             output_dict = json.loads(output_ptr.value[:output_len.value].decode("utf-8"))
@@ -82,17 +85,21 @@ class Query:
 
 
 class DjangoMiddleware:
-    get_response: Callable
-    path: str = '/graphql'      # default value
-    jwt: str = 'authorization'  # authorization header name, jwt expected
-
-    def __init__(self, get_response: Callable):
+    def __init__(self, get_response):
         # save response processing fn
         self.get_response = get_response
 
+        if ffi.library is None:
+            # library is not found, skip middleware initialization
+            return
+
+        # default values
+        self.path = '/graphql'
+        self.jwt = 'authorization'  # authorization header name, jwt expected
+
         c = ffi.Config()
 
-        inigo_settings: Dict = {}
+        inigo_settings = {}
 
         if hasattr(settings, 'INIGO'):
             inigo_settings = settings.INIGO
@@ -111,13 +118,15 @@ class DjangoMiddleware:
         schema = None
         if inigo_settings.get('GRAPHENE_SCHEMA'):
             schema = import_string(inigo_settings.get('GRAPHENE_SCHEMA'))
-        else:
-            if hasattr(settings, 'GRAPHENE') and settings.GRAPHENE.get('SCHEMA'):
-                schema = import_string(settings.GRAPHENE.get('SCHEMA'))
+        elif inigo_settings.get('SCHEMA_PATH'):
+            if os.path.isfile(inigo_settings.get('SCHEMA_PATH')):
+                with open(inigo_settings.get('SCHEMA_PATH'), 'r') as f:
+                    schema = f.read()
+        elif hasattr(settings, 'GRAPHENE') and settings.GRAPHENE.get('SCHEMA'):
+            schema = import_string(settings.GRAPHENE.get('SCHEMA'))
 
         if schema:
             c.schema = str.encode(str(schema))
-            c.introspection = b'{ "data": %s }' % str.encode(str(json.dumps(schema.introspect())))
 
         if inigo_settings.get('PATH'):
             self.path = inigo_settings.get('PATH')
@@ -127,32 +136,39 @@ class DjangoMiddleware:
 
         # create Inigo instance
         self.instance = ffi.create(ctypes.byref(c))
+
+        error = ffi.check_lasterror()
+        if error:
+            raise Exception(error.decode('utf-8'))
+
         if self.instance == 0:
             raise Exception('error, instance can not be created')
 
-    def __call__(self, request: HttpRequest) -> HttpResponse:
+    def __call__(self, request):
+        # ignore execution if Inigo is not initialized
+        if not self.instance:
+            return self.get_response(request)
+
         # 'path' guard -> /graphql
         if request.path != self.path:
             return self.get_response(request)
 
-        # read request from body
-        query: str = ''
+        if request.method != 'POST':
+            return self.get_response(request)
 
-        if request.method == 'POST':
-            query = json.loads(request.body).get('query')
-        elif request.method == 'GET':
-            query = request.GET.get('query')
+        # read request from body
+        query = json.loads(request.body).get('query')
 
         q = Query(self.instance, query)
 
         # create inigo context if not present. Should exist before 'get_auth_token' call
-        if request.inigo is None:
+        if hasattr(request, 'inigo') is False:
             request.inigo = InigoContext()
 
-        auth = self.get_auth_token(self.jwt, request)
+        token = self.get_auth_token(self.jwt, request)
 
         # inigo: process request
-        output, status = q.process_request(auth)
+        output, status = q.process_request(token)
 
         # introspection query
         if output and output.get('data') and output.get('data').get('__schema'):
@@ -186,7 +202,11 @@ class DjangoMiddleware:
         # forward to request handler
         response = self.get_response(request)
 
-        if response.headers.get('content-type') != 'application/json':
+        # return if response is not json
+        try:
+            _ = orjson.loads(response.content)
+        except ValueError:  # includes simplejson.decoder.JSONDecodeError
+            # cannot parse json
             q.ingest()
 
             return response
@@ -199,9 +219,9 @@ class DjangoMiddleware:
         return response
 
     @staticmethod
-    def get_auth_token(header: str, request: HttpRequest) -> str:
+    def get_auth_token(header, request):
         if hasattr(request, 'inigo') and isinstance(request.inigo, InigoContext) is False:
-            raise Exception("'inigo' attr is not InigoContext instance")
+            raise Warning("'inigo' attr is not InigoContext instance")
 
         # read from request object
         if hasattr(request, 'inigo') and isinstance(request.inigo, InigoContext) and request.inigo.auth:
@@ -212,7 +232,7 @@ class DjangoMiddleware:
             return request.headers.get(header)
 
     @staticmethod
-    def respond(data: Dict) -> JsonResponse:
+    def respond(data):
         response = {
             'data': data.get('data'),
         }
@@ -227,15 +247,16 @@ class DjangoMiddleware:
 
 
 class InigoContext:
-    __auth: Dict | None = None
-    __blocked: bool = False
+    def __init__(self):
+        self.__auth = None
+        self.__blocked = False
 
     @property
     def auth(self):
         return self.__auth
 
     @auth.setter
-    def auth(self, value: Dict):
+    def auth(self, value):
         self.__auth = value
 
     @property
